/*************************************************
*     Exim - an Internet mail transport agent    *
*************************************************/

/* Copyright (c) University of Cambridge 1995 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

/* This file, which provides support for Microsoft's Secure Password
Authentication, was contributed by Marc Prud'hommeaux. */


#include "../exim.h"
#include "spa.h"

/* #define DEBUG_SPA */

#ifdef DEBUG_SPA
#define DSPA(x,y,z)   debug_printf(x,y,z)
#else
#define DSPA(x,y,z)
#endif

/* Options specific to the spa authentication mechanism. */

optionlist auth_spa_options[] = {
  { "client_domain",             opt_stringptr,
      (void *)(offsetof(auth_spa_options_block, spa_domain)) },
  { "client_password",           opt_stringptr,
      (void *)(offsetof(auth_spa_options_block, spa_password)) },
  { "client_username",           opt_stringptr,
      (void *)(offsetof(auth_spa_options_block, spa_username)) },
};

/* Size of the options list. An extern variable has to be used so that its
address can appear in the tables drtables.c. */

int auth_spa_options_count =
  sizeof(auth_spa_options)/sizeof(optionlist);

/* Default private options block for the contidion authentication method. */

auth_spa_options_block auth_spa_option_defaults = {
  NULL,              /* spa_password */
  NULL,              /* spa_username */
  0                  /* spa_domain */
};


/*************************************************
*          Initialization entry point            *
*************************************************/

/* Called for each instance, after its options have been read, to
enable consistency checks to be done, or anything else that needs
to be set up. */

void
auth_spa_init(auth_instance *ablock)
{
/*
auth_spa_options_block *ob =
  (auth_spa_options_block *)(ablock->options_block);
*/

if (ablock->public_name == NULL) ablock->public_name = ablock->name;

/* server is disabled for SPA */
ablock->server = FALSE;

ablock->client = TRUE;
}



/*************************************************
*             Server entry point                 *
*************************************************/

/* For interface, see auths/README */

int
auth_spa_server(auth_instance *ablock, uschar *data)
{
  /* suppress compiler warning */
  data = data;
  /* the server is not supported for SPA */
  debug_printf("%s authenticator: server not supported for this authentication scheme\n", ablock->name);
  return FAIL;
}


/*************************************************
*              Client entry point                *
*************************************************/

/* For interface, see auths/README */

int
auth_spa_client(
  auth_instance *ablock,                 /* authenticator block */
  smtp_inblock *inblock,                 /* connection inblock */
  smtp_outblock *outblock,               /* connection outblock */
  int timeout,                           /* command timeout */
  uschar *buffer,                        /* buffer for reading response */
  int buffsize)                          /* size of buffer */
{
       auth_spa_options_block *ob =
	       (auth_spa_options_block *)(ablock->options_block);
       SPAAuthRequest	request;
       SPAAuthChallenge challenge;
       SPAAuthResponse	response;
       char msgbuf[2048];
       char *domain;

    if (!smtp_write_command(outblock, FALSE, "AUTH %s\r\n",
         ablock->public_name))
	       return FAIL_SEND;

       /* wait for the 3XX OK message */
       if (!smtp_read_response(inblock, (uschar *)buffer, buffsize, '3', timeout))
	       return FAIL;

       /* last-ditch sanity checks */
       if (ob->spa_username == NULL)
       {
	       debug_printf("%s authenticator: username unset\n", ablock->name);
	       return ERROR;
       }

       if (ob->spa_password == NULL)
       {
	       debug_printf("%s authenticator: password unset\n", ablock->name);
	       return ERROR;
       }

       domain = CS(ob->spa_domain);
       DSPA("\n\n%s authenticator: using domain %s\n\n",
	       ablock->name, ob->spa_domain);

       spa_build_auth_request (&request, CS(ob->spa_username), domain);
       spa_bits_to_base64 (US msgbuf, (unsigned char*)&request,
	       spa_request_length(&request));

       DSPA("\n\n%s authenticator: sending request (%s)\n\n", ablock->name,
	       msgbuf);

       /* send the encrypted password */
    if (!smtp_write_command(outblock, FALSE, "%s\r\n", msgbuf))
	       return FAIL_SEND;

       /* wait for the auth challenge */
       if (!smtp_read_response(inblock, (uschar *)buffer, buffsize, '3', timeout))
	       return FAIL;

       /* convert the challenge into the challenge struct */
       DSPA("\n\n%s authenticator: challenge (%s)\n\n",
	       ablock->name, buffer + 4);
       spa_base64_to_bits ((char *)(&challenge), (const char *)(buffer + 4));

       spa_build_auth_response (&challenge, &response,
	       CS(ob->spa_username), CS(ob->spa_password));
       spa_bits_to_base64 (US msgbuf, (unsigned char*)&response,
	       spa_request_length(&response));
       DSPA("\n\n%s authenticator: challenge response (%s)\n\n", ablock->name,
	       msgbuf);

       /* send the challenge response */
    if (!smtp_write_command(outblock, FALSE, "%s\r\n", msgbuf))
	       return FAIL_SEND;

       /* If we receive a success response from the server, authentication
       has succeeded. There may be more data to send, but is there any point
       in provoking an error here? */
       if (smtp_read_response(inblock, US buffer, buffsize, '2', timeout))
	       return OK;

       /* Not a success response. If errno != 0 there is some kind of transmission
       error. Otherwise, check the response code in the buffer. If it starts with
       '3', more data is expected. */
       if (errno != 0 || buffer[0] != '3')
	       return FAIL;

       return FAIL;
}

/* End of spa.c */
