/*************************************************
*     Exim - an Internet mail transport agent    *
*************************************************/

/* Copyright (c) University of Cambridge 1995 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

/* Functions concerned with verifying things. */


#include "exim.h"


/* Structure for passing arguments to check_host() */

typedef struct check_host_block {
  uschar *host_name;
  uschar *host_address;
  uschar *host_ipv4;
} check_host_block;


/* Structure for caching DNSBL lookups */

typedef struct dnsbl_cache_block {
  dns_address *rhs;
  uschar *text;
  int rc;
  BOOL text_set;
} dnsbl_cache_block;


/* Anchor for DNSBL cache */

static tree_node *dnsbl_cache = NULL;



/*************************************************
*            Verify an email address             *
*************************************************/

/* This function is used both for verification (-bv and at other times) and
address testing (-bt), which is indicated by address_test_mode being set.

Arguments:
  addr            contains the address to verify
  f               if not NULL, write the result to this file
  options         various option bits:
                    vopt_is_recipient => this is a recipient address, otherwise
                      it's a sender address - this affects qualification and
                      rewriting
                    vopt_qualify => qualify an unqualified address; else error
                    vopt_expn => called from SMTP EXPN command
  callout         if > 0, specifies that callout is required, and gives timeout

Returns:          OK      address verified
                  FAIL    address failed to verify
                  DEFER   can't tell at present
*/

int
verify_address(address_item *addr, FILE *f, int options, int callout)
{
BOOL allok = TRUE;
BOOL full_info = (f == NULL)? FALSE : (debug_selector != 0);
BOOL is_recipient = (options & vopt_is_recipient) != 0;
BOOL expn         = (options & vopt_expn) != 0;
int i;
int yield = OK;
int verify_type = expn? v_expn :
     address_test_mode? v_none :
          is_recipient? v_recipient : v_sender;
address_item *addr_list;
address_item *addr_new = NULL;
address_item *addr_remote = NULL;
address_item *addr_local = NULL;
address_item *addr_succeed = NULL;
uschar *ko_prefix, *cr;
uschar *address = addr->orig;

/* Set up a prefix and suffix for error message which allow us to use the same
output statements both in EXPN mode (where an SMTP response is needed) and when
debugging with an output file. */

if (expn)
  {
  ko_prefix = US"553 ";
  cr = US"\r";
  }
else ko_prefix = cr = US"";

/* Add qualify domain if permitted; otherwise an unqualified address fails. */

if (parse_find_at(address) == NULL)
  {
  if ((options & vopt_qualify) == 0)
    {
    if (f != NULL)
      fprintf(f, "%sA domain is required for \"%s\"%s\n", ko_prefix, address,
        cr);
    return FAIL;
    }
  address = rewrite_address_qualify(address, is_recipient);
  }

DEBUG(D_verify)
  {
  debug_printf(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>\n");
  debug_printf("%s %s\n", address_test_mode? "Testing" : "Verifying", address);
  }

/* Rewrite and report on it. */

if (global_rewrite_rules != NULL)
  {
  uschar *old = address;
  address = rewrite_address(address, is_recipient, FALSE,
    global_rewrite_rules, rewrite_existflags);
  if (address != old && f != NULL && !expn)
    fprintf(f, "Address rewritten as: %s\n", address);
  }

/* Update the address structure with the possibly qualified and rewritten
address. Set it up as the starting address on the chain of new addresses. */

addr->orig = addr->rcpt = addr->active = address;
addr_new = addr;

/* We need a loop, because an address can generate new addresses. We must also
cope with generated pipes and files at the top level. (See also the code and
comment in deliver.c.) However, it is usually the case that the router for
user's .forward files has its verify flag turned off.

If an address generates more than one child, the loop is used only when
full_info is set, and this can only be set locally. Remote enquiries just get
information about the top level address, not anything that it generated. */

while (addr_new != NULL)
  {
  int rc;
  addr = addr_new;
  addr_new = addr->next;
  addr->next = NULL;

  DEBUG(D_verify)
    {
    debug_printf(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>\n");
    debug_printf("Considering %s\n", addr->active);
    }

  /* Handle generated pipe, file or reply addresses. We don't get these
  when handling EXPN, as it does only one level of expansion. */

  if (testflag(addr, af_pfr))
    {
    allok = FALSE;
    if (f != NULL)
      {
      BOOL allow;

      if (addr->orig[0] == '>')
        {
        allow = testflag(addr, af_allow_reply);
        fprintf(f, "%s -> mail %s", addr->parent->orig, addr->orig + 1);
        }
      else
        {
        allow = (addr->orig[0] == '|')?
          testflag(addr, af_allow_pipe) : testflag(addr, af_allow_file);
        fprintf(f, "%s -> %s", addr->parent->orig, addr->orig);
        }

      if (addr->basic_errno == ERRNO_BADTRANSPORT)
        fprintf(f, "\n*** Error in setting up pipe, file, or autoreply:\n"
          "%s\n", addr->message);
      else if (allow)
        fprintf(f, "\n  transport = %s\n", addr->transport->name);
      else
        fprintf(f, " *** forbidden ***\n");
      }
    continue;
    }


  /* Split the address into domain and local part, handling the %-hack if
  necessary, and then route it. */

  if ((rc = deliver_split_address(addr)) == OK)
    rc = route_address(addr, &addr_local, &addr_remote, &addr_new,
      &addr_succeed, verify_type);

  /* If routing an address succeeded and set up a list of hosts, and the
  callout option is set, do the full callout verification. This is an
  expensive operation. */

  if (rc == OK && addr->host_list != NULL && callout > 0)
    {
    host_item *host;
    BOOL done = FALSE;

    HDEBUG(D_verify)
      debug_printf("Attempting full verification using callout\n");

    if (host_checking)
      {
      HDEBUG(D_verify) debug_printf("... callout omitted when host testing\n");
      done = TRUE;
      }

    else for (host = addr->host_list;
              host != NULL && ! done;
              host = host->next)
      {
      smtp_inblock inblock;
      smtp_outblock outblock;
      int host_af;
      int port = 25;
      uschar *interface = NULL;  /* Outgoing interface to use; NULL => any */
      uschar inbuffer[1024];
      uschar outbuffer[1024];
      uschar responsebuffer[1024];

      /* Skip this host if we don't have an IP address for it. */

      if (host->address == NULL) continue;

      /* Set IPv4 or IPv6 */

      host_af = (Ustrchr(host->address, ':') == NULL)? AF_INET:AF_INET6;

      /* If verification yielded a remote transport, we want to use that
      transport's interface and port options, so as to mimic what would
      happen if we were really sending a message to this address. If there is
      a problem getting the values, log a panic message, but carry on with
      the default values. */

      if (addr->transport != NULL && !addr->transport->info->local)
        {
        uschar *ifstring = NULL;
        uschar *portstring = NULL;

        (void)(addr->transport->setup)(addr->transport, addr, &ifstring,
          &portstring, NULL);

        deliver_host = host->name;
        deliver_host_address = host->address;
        if (!smtp_get_interface(ifstring, host_af, addr, NULL, &interface,
                US"callout") ||
            !smtp_get_port(portstring, addr, &port, US"callout"))
          log_write(0, LOG_MAIN|LOG_PANIC, "<%s>: %s", addr->orig,
            addr->message);
        deliver_host = deliver_host_address = NULL;
        }

      HDEBUG(D_verify) debug_printf("interface=%s port=%d\n", interface, port);

      /* Set up the buffer for reading SMTP response packets. */

      inblock.buffer = inbuffer;
      inblock.buffersize = sizeof(inbuffer);
      inblock.ptr = inbuffer;
      inblock.ptrend = inbuffer;

      /* Set up the buffer for holding SMTP commands while pipelining */

      outblock.buffer = outbuffer;
      outblock.buffersize = sizeof(outbuffer);
      outblock.ptr = outbuffer;
      outblock.cmd_count = 0;

      /* Connect to the host; on failure, just loop for the next one */

      inblock.sock = outblock.sock =
        smtp_connect(host, host_af, port, interface, callout, TRUE);
      if (inblock.sock < 0) continue;

      /* Wait for initial response, and then run some SMTP commands. The
      smtp_write_command() function leaves its command in big_buffer. This is
      used in error responses. Initialize it in case the connection is
      rejected. */

      Ustrcpy(big_buffer, "initial connection");

      if (smtp_read_response(&inblock, responsebuffer, sizeof(responsebuffer),
            '2', callout) &&

          smtp_write_command(&outblock, FALSE, "HELO %s\r\n",
            primary_hostname) >= 0 &&
          smtp_read_response(&inblock, responsebuffer, sizeof(responsebuffer),
            '2', callout) &&

          smtp_write_command(&outblock, FALSE, "MAIL FROM:<>\r\n") >= 0 &&
          smtp_read_response(&inblock, responsebuffer, sizeof(responsebuffer),
            '2', callout) &&

          smtp_write_command(&outblock, FALSE, "RCPT TO:<%.1000s>\r\n",
            addr->rcpt) >= 0 &&
          smtp_read_response(&inblock, responsebuffer, sizeof(responsebuffer),
            '2', callout))
        {
        done = TRUE;                /* Address accepted */
        }

      /* For any failure other than a negative response, we just close the
      connection and carry on. We can identify a negative response by the fact
      that errno is zero. For I/O errors it will be non-zero

      Set up different error texts for logging and for sending back to the
      caller as an SMTP response. Log in all cases, using a one-line format.
      For sender callouts, give a full response to the caller, but for
      recipient callouts, don't give the IP address because this may be an
      internal host whose identity is not to be widely broadcast. */

      else if (errno == 0)
        {
        addr->message =
          string_sprintf("response to \"%s\" from %s [%s] was %s",
            big_buffer, host->name, host->address,
            string_printing(responsebuffer));

        addr->user_message = is_recipient?
          string_sprintf("Callout verification failed:\n%s", responsebuffer)
          :
          string_sprintf("Called:   %s\nSent:     %s\nResponse: %s",
            host->address, big_buffer, responsebuffer);

        /* Hard rejection ends the process */

        if (responsebuffer[0] == '5')   /* Address rejected */
          {
          rc = FAIL;
          done = TRUE;
          }
        }

      /* End the SMTP conversation and close the connection. */

      smtp_write_command(&outblock, FALSE, "QUIT\r\n");
      close(inblock.sock);
      }    /* Loop through all hosts, while !done */

    /* Failure to connect to any host, or any response other than 2xx or 5xx
    is a temporary error. If there was only one host, and a response was
    received, leave it alone. Otherwise, give a generic response. */

    if (!done)
      {
      rc = DEFER;
      if (addr->host_list->next != NULL || addr->message == NULL)
        addr->user_message = addr->message =
          string_sprintf("Could not complete %s callout check",
            is_recipient? "recipient" : "sender");
      addr->basic_errno = ERRNO_CALLOUTDEFER;
      }
    }   /* Callout */

  /* A router may return REROUTE if it has changed the value of addr->domain
  and addr->rcpt. This can happen when a domain routes to the local host, and
  the "self" setting specifies rerouting. It can also happen if an abbreviated
  domain gets expanded by DNS lookup, and the expanded domain routes to the
  local host. */

  if (rc == REROUTE)
    {
    addr->host_list = NULL;
    addr->next = addr_new;
    addr_new = addr;
    continue;
    }

  /* Handle hard failures */

  if (rc == FAIL)
    {
    allok = FALSE;
    if (f != NULL)
      {
      fprintf(f, "%s%s %s", ko_prefix, addr->active,
        address_test_mode? "is undeliverable" : "failed to verify");
      if (!expn)
        {
        if (addr->basic_errno > 0)
          fprintf(f, ": %s", strerror(addr->basic_errno));
        if (addr->message != NULL)
          fprintf(f, ":%s%s", expn? " " : "\n  ", addr->message);
        }
      fprintf(f, "%s\n", cr);
      }
    if (!full_info) return FAIL; else yield = FAIL;
    }

  /* Soft failure */

  else if (rc == DEFER)
    {
    allok = FALSE;
    if (f != NULL)
      {
      fprintf(f, "%s%s cannot be resolved at this time", ko_prefix,
        addr->orig);
      if (!expn)
        fprintf(f, ":\n  %s%s%s",
          (addr->basic_errno <= 0)? "" : strerror(addr->basic_errno),
          (addr->basic_errno <= 0)? "" : ": ",
          (addr->message != NULL)? addr->message :
            (addr->basic_errno <= 0)? US"unknown error" : US"");
      fprintf(f, "%s\n", cr);
      }
    if (!full_info) return DEFER;
      else if (yield == OK) yield = DEFER;
    }

  /* If we are handling EXPN, we do not want to continue to route beyond
  the top level. */

  else if (expn)
    {
    uschar *ok_prefix = US"250-";
    if (addr_new == NULL)
      {
      if (addr_local == NULL && addr_remote == NULL)
        fprintf(f, "250 mail to <%s> is discarded\r\n", address);
      else
        fprintf(f, "250 <%s>\r\n", addr->orig);
      }
    else while (addr_new != NULL)
      {
      address_item *addr2 = addr_new;
      addr_new = addr2->next;
      if (addr_new == NULL) ok_prefix = US"250 ";
      fprintf(f, "%s<%s>\r\n", ok_prefix, addr2->orig);
      }
    return OK;
    }

  /* Successful routing other than EXPN. */

  else
    {
    /* Handle successful routing when short info wanted. Otherwise continue for
    other (generated) addresses.

    There is a conflict between the use of aliasing as an alternate email
    address, and as a sort of mailing list. If an alias turns the incoming
    address into just one address (e.g. J.Caesar->jc44) you may well want to
    carry on verifying the generated address to ensure it is valid when
    checking incoming mail. If aliasing generates multiple addresses, you
    probably don't want to do this. Exim therefore treats the generation of
    just a single new address as a special case, and continues on to verify the
    generated address. */

    if (!full_info &&                    /* Stop if short info wanted AND */
         (addr_new == NULL ||            /* No new address OR */
          addr_new->next != NULL ||      /* More than one new address OR */
          testflag(addr_new, af_pfr)))   /* New address is pfr */
      {
      if (f != NULL) fprintf(f, "%s %s\n", addr->orig,
        address_test_mode? "is deliverable" : "verified");
      return OK;
      }
    }
  }     /* Loop for generated addresses */

/* Display the full results of the successful routing, including any generated
addresses. Control gets here only when full_info is set, which requires f not
to be NULL, and this occurs only when a top-level verify is called with the
debugging switch on.

If there are no local and no remote addresses, and there were no pipes, files,
or autoreplies, and there were no errors or deferments, the message is to be
discarded, usually because of the use of :blackhole: in an alias file. */

if (allok && addr_local == NULL && addr_remote == NULL)
  fprintf(f, "mail to %s is discarded\n", address);

else for (addr_list = addr_local, i = 0; i < 2; addr_list = addr_remote, i++)
  {
  while (addr_list != NULL)
    {
    address_item *addr = addr_list;
    address_item *p = addr->parent;
    addr_list = addr->next;

    fprintf(f, "%s", addr->orig);
    while (p != NULL)
      {
      fprintf(f, "\n    <-- %s", p->orig);
      p = p->parent;
      }
    fprintf(f, "\n  ");
    if (Ustrcmp(addr->orig, addr->rcpt) != 0)
      fprintf(f, "envelope to: %s\n  ", addr->rcpt);

    /* Show router, and transport */

    fprintf(f, "router = %s, ", addr->router->name);
    fprintf(f, "transport = %s\n", (addr->transport == NULL)? US"unset" :
      addr->transport->name);

    /* Show any hosts that are set up by a router unless the transport
    is going to override them; fiddle a bit to get a nice format. */

    if (addr->host_list != NULL && addr->transport != NULL &&
        !addr->transport->overrides_hosts)
      {
      host_item *h;
      int maxlen = 0;
      int maxaddlen = 0;
      for (h = addr->host_list; h != NULL; h = h->next)
        {
        int len = Ustrlen(h->name);
        if (len > maxlen) maxlen = len;
        len = (h->address != NULL)? Ustrlen(h->address) : 7;
        if (len > maxaddlen) maxaddlen = len;
        }
      for (h = addr->host_list; h != NULL; h = h->next)
        {
        int len = Ustrlen(h->name);
        fprintf(f, "  host %s ", h->name);
        while (len++ < maxlen) fprintf(f, " ");
        if (h->address != NULL)
          {
          fprintf(f, "[%s] ", h->address);
          len = Ustrlen(h->address);
          }
        else if (!addr->transport->info->local)  /* Omit [unknown] for local */
          {
          fprintf(f, "[unknown] ");
          len = 7;
          }
        else len = -3;
        while (len++ < maxaddlen) fprintf(f," ");
        if (h->mx >= 0) fprintf(f, "MX=%d", h->mx);
        if (h->status == hstatus_unusable) fprintf(f, " ** unusable **");
        fprintf(f, "\n");
        }
      }
    }
  }

return yield;  /* Will be DEFER or FAIL if any one address has */
}




/*************************************************
*      Check headers for syntax errors           *
*************************************************/

/* This function checks those header lines that contain addresses, and verifies
that all the addresses therein are syntactially correct.

Arguments:
  msgptr     where to put an error message

Returns:     OK
             FAIL
*/

int
verify_check_headers(uschar **msgptr)
{
header_line *h;
uschar *colon, *s;

for (h = header_list; h != NULL; h = h->next)
  {
  if (h->type != htype_from &&
      h->type != htype_reply_to &&
      h->type != htype_sender &&
      h->type != htype_to &&
      h->type != htype_cc &&
      h->type != htype_bcc)
    continue;

  colon = Ustrchr(h->text, ':');
  s = colon + 1;
  while (isspace(*s)) s++;

  parse_allow_group = TRUE;     /* Allow group syntax */

  /* Loop for multiple addresses in the header */

  while (*s != 0)
    {
    uschar *ss = parse_find_address_end(s, FALSE);
    uschar *recipient, *errmess;
    int terminator = *ss;
    int start, end, domain;

    /* Temporarily terminate the string at this point, and extract the
    operative address within. */

    *ss = 0;
    recipient = parse_extract_address(s,&errmess,&start,&end,&domain,FALSE);
    *ss = terminator;

    if (recipient == NULL && Ustrcmp(errmess, "empty address") != 0)
      {
      uschar hname[64];
      uschar *t = h->text;
      uschar *tt = hname;
      uschar *verb = US"is";
      int len;

      while (*t != ':') *tt++ = *t++;
      *tt = 0;

      /* Arrange not to include any white space at the end in the
      error message. */

      t = ss;
      while (t > s && isspace(t[-1])) t--;

      /* Add the address which failed to the error message, since in a
      header with very many addresses it is sometimes hard to spot
      which one is at fault. However, limit the amount of address to
      quote - cases have been seen where, for example, a missing double
      quote in a humungous To: header creates an "address" that is longer
      than string_sprintf can handle. */

      len = t - s;
      if (len > 1024)
        {
        len = 1024;
        verb = US"begins";
        }

      *msgptr = string_printing(
        string_sprintf("%s: failing address in \"%.*s\" header %s: %.*s",
          errmess, colon - h->text, h->text, verb, len, s));

      return FAIL;
      }

    /* Advance to the next address */

    s = ss + (terminator? 1:0);
    while (isspace(*s)) s++;
    }   /* Next address */
  }     /* Next header */

return OK;
}




/*************************************************
*          Find if verified sender               *
*************************************************/

/* Usually, just a single address is verified as the sender of the message.
However, Exim can be made to verify other addresses as well (often related in
some way), and this is useful in some environments. There may therefore be a
chain of such addresses that have previously been tested. This function finds
whether a given address is on the chain.

Arguments:   the address to be verified
Returns:     pointer to an address item, or NULL
*/

address_item *
verify_checked_sender(uschar *sender)
{
address_item *addr;
for (addr = sender_verified_list; addr != NULL; addr = addr->next)
  if (Ustrcmp(sender, addr->orig) == 0) break;
return addr;
}





/*************************************************
*             Get valid header address           *
*************************************************/

/* Scan the originator headers of the message, looking for an address that
verifies successfully. RFC 822 says:

    o   The "Sender" field mailbox should be sent  notices  of
        any  problems in transport or delivery of the original
        messages.  If there is no  "Sender"  field,  then  the
        "From" field mailbox should be used.

    o   If the "Reply-To" field exists, then the reply  should
        go to the addresses indicated in that field and not to
        the address(es) indicated in the "From" field.

So we check a Sender field if there is one, else a Reply_to field, else a From
field. As some strange messages may have more than one of these fields,
especially if they are resent- fields, check all of them if there is more than
one.

Arguments:
  error_ptr     points to where to put an error message
  callout       timeout for callout check (passed to verify_address())

Returns:        result of the verification attempt: OK, FAIL, or DEFER;
                FAIL is given if no appropriate headers are found
*/

int
verify_check_header_address(uschar **parse_error_ptr, int callout)
{
static int header_types[] = { htype_sender, htype_reply_to, htype_from };
int yield = FAIL;
int i;

for (i = 0; i < 3; i++)
  {
  header_line *h;
  for (h = header_list; h != NULL; h = h->next)
    {
    int terminator, new_ok;
    uschar *s, *ss, *endname;

    if (h->type != header_types[i]) continue;
    s = endname = Ustrchr(h->text, ':') + 1;

    while (*s != 0)
      {
      address_item *vaddr;

      while (isspace(*s) || *s == ',') s++;
      if (*s == 0) break;        /* End of header */

      ss = parse_find_address_end(s, FALSE);

      /* The terminator is a comma or end of header, but there may be white
      space preceding it (including newline for the last address). Move back
      past any white space so we can check against any cached envelope sender
      address verifications. */

      while (isspace(ss[-1])) ss--;
      terminator = *ss;
      *ss = 0;

      HDEBUG(D_verify) debug_printf("verifying %.*s header address %s\n",
        endname - h->text, h->text, s);

      /* See if we have already verified this address as an envelope sender,
      and if so, use the previous answer. */

      vaddr = verify_checked_sender(s);

      if (vaddr != NULL &&                   /* Previously checked */
           (callout <= 0 ||                  /* No callout needed; OR */
            vaddr->special_action > 256))    /* Callout was done */
        {
        new_ok = vaddr->special_action & 255;
        HDEBUG(D_verify) debug_printf("previously checked as envelope sender\n");
        }
      else
        {
        int start, end, domain;
        uschar *address = parse_extract_address(s, parse_error_ptr, &start,
          &end, &domain, FALSE);
        if (address == NULL)
          new_ok = FAIL;
        else
          {
          vaddr = deliver_make_addr(address, FALSE);
            /* We must restore the shortened string before running the
            verification, so the headers are correct, in case there is any
            rewriting. */
          *ss = terminator;
          new_ok = verify_address(vaddr, NULL, 0, callout);
          }
        }

      *ss = terminator;  /* This catches all the other cases */

      /* If verification failed because of a syntax error, fail this function,
      and ensure that the failing address gets added to the error message. */

      if (new_ok == FAIL && *parse_error_ptr != NULL)
        {
        while (ss > s && isspace(ss[-1])) ss--;
        *parse_error_ptr =
          string_sprintf("syntax error in '%.*s' header when scanning for "
            "sender: %s in \"%.*s\"",
            endname - h->text, h->text, *parse_error_ptr, ss - s, s);
        return FAIL;
        }

      /* Success or defer */

      if (new_ok == OK) return OK;
      if (new_ok == DEFER) yield = DEFER;

      /* Move on to any more addresses in the header */

      s = ss;
      }
    }
  }

return yield;
}




/*************************************************
*            Get RFC 1413 identification         *
*************************************************/

/* Attempt to get an id from the sending machine via the RFC 1413 protocol. If
the timeout is set to zero, then the query is not done. There may also be lists
of hosts and nets which are exempt. To guard against malefactors sending
non-printing characters which could, for example, disrupt a message's headers,
make sure the string consists of printing characters only.

Argument:
  port    the port to connect to; usually this is IDENT_PORT (113), but when
          running in the test harness with -bh a different value is used.

Returns:  nothing

Side effect: any received ident value is put in sender_ident (NULL otherwise)
*/

void
verify_get_ident(int port)
{
int sock, host_af, qlen;
int received_sender_port, received_interface_port, n;
uschar *p;
uschar buffer[2048];

/* Default is no ident. Check whether we want to do an ident check for this
host. */

sender_ident = NULL;
if (rfc1413_query_timeout <= 0 || verify_check_host(&rfc1413_hosts) != OK)
  return;

DEBUG(D_ident) debug_printf("doing ident callback\n");

/* Set up a connection to the ident port of the remote host. Bind the local end
to the incoming interface address. If the sender host address is an IPv6
address, the incoming interface address will also be IPv6. */

host_af = (Ustrchr(sender_host_address, ':') == NULL)? AF_INET : AF_INET6;
sock = ip_socket(SOCK_STREAM, host_af);
if (sock < 0) return;

if (ip_bind(sock, host_af, interface_address, 0) < 0)
  {
  DEBUG(D_ident) debug_printf("bind socket for ident failed: %s\n",
    strerror(errno));
  goto END_OFF;
  }

if (ip_connect(sock, host_af, sender_host_address, port, rfc1413_query_timeout)
     < 0)
  {
  DEBUG(D_ident) debug_printf("ident connection to %s failed: %s\n",
    sender_host_address, strerror(errno));
  goto END_OFF;
  }

/* Construct and send the query. */

sprintf(CS buffer, "%d , %d\r\n", sender_host_port, interface_port);
qlen = Ustrlen(buffer);
if (send(sock, buffer, qlen, 0) < 0)
  {
  DEBUG(D_ident) debug_printf("ident send failed: %s\n", strerror(errno));
  goto END_OFF;
  }

/* Read a response line. We put it into the rest of the buffer, using several
recv() calls if necessary. */

p = buffer + qlen;

for (;;)
  {
  uschar *pp;
  int count;
  int size = sizeof(buffer) - (p - buffer);

  if (size <= 0) goto END_OFF;   /* Buffer filled without seeing \n. */
  count = ip_recv(sock, p, size, rfc1413_query_timeout);
  if (count <= 0) goto END_OFF;  /* Read error or EOF */

  /* Scan what we just read, to see if we have reached the terminating \r\n. Be
  generous, and accept a plain \n terminator as well. The only illegal
  character is 0. */

  for (pp = p; pp < p + count; pp++)
    {
    if (*pp == 0) goto END_OFF;   /* Zero octet not allowed */
    if (*pp == '\n')
      {
      if (pp[-1] == '\r') pp--;
      *pp = 0;
      goto GOT_DATA;             /* Break out of both loops */
      }
    }

  /* Reached the end of the data without finding \n. Let the loop continue to
  read some more, if there is room. */

  p = pp;
  }

GOT_DATA:

/* We have received a line of data. Check it carefully. It must start with the
same two port numbers that we sent, followed by data as defined by the RFC. For
example,

  12345 , 25 : USERID : UNIX :root

However, the amount of white space may be different to what we sent. In the
"osname" field there may be several sub-fields, comma separated. The data we
actually want to save follows the third colon. Some systems put leading spaces
in it - we discard those. */

if (sscanf(CS buffer + qlen, "%d , %d%n", &received_sender_port,
      &received_interface_port, &n) != 2 ||
    received_sender_port != sender_host_port ||
    received_interface_port != interface_port)
  return;

p = buffer + qlen + n;
while(isspace(*p)) p++;
if (*p++ != ':') return;
while(isspace(*p)) p++;
if (Ustrncmp(p, "USERID", 6) != 0) return;
p += 6;
while(isspace(*p)) p++;
if (*p++ != ':') return;
while (*p != 0 && *p != ':') p++;
if (*p++ == 0) return;
while(isspace(*p)) p++;
if (*p == 0) return;

/* The rest of the line is the data we want. We turn it into printing
characters when we save it, so that it cannot mess up the format of any logging
or Received: lines into which it gets inserted. We keep a maximum of 127
characters. */

sender_ident = string_printing(string_copyn(p, 127));
DEBUG(D_ident) debug_printf("sender_ident = %s\n", sender_ident);

END_OFF:
close(sock);
return;
}




/*************************************************
*      Match host to a single host-list item     *
*************************************************/

/* This function compares a host (name or address) against a single item
from a host list. The host name gets looked up if it is needed and is not
already known. The function is called from verify_check_this_host() via
match_check_list(), which is why most of its arguments are in a single block.

Arguments:
  arg            the argument block (see below)
  ss             the host-list item

The block contains:
  host_name      the host name or NULL, implying use sender_host_name and
                   sender_host_aliases, looking them up if required
  host_address   the host address
  host_ipv4      the IPv4 address taken from an IPv6 one

Returns:         OK      matched
                 FAIL    did not match
                 DEFER   lookup deferred
                 ERROR   failed to find the host name
*/

static int
check_host(void *arg, uschar *ss)
{
check_host_block *cb = (check_host_block *)arg;
int maskoffset;
uschar *semicolon, *t;
uschar **aliases;

/* Optimize for the special case when the pattern is "*". */

if (*ss == '*' && ss[1] == 0) return OK;

/* If the pattern is empty, it matches only in the case when there is no host -
this can occur in ACL checking for SMTP input using the -bs option. In this
situation, the host address is the empty string. */

if (cb->host_address[0] == 0) return (*ss == 0)? OK : FAIL;
if (*ss == 0) return FAIL;

/* If the pattern is precisely "@" then match against the primary host name;
if it's "@[]" match against the local host's IP addresses. */

if (*ss == '@')
  {
  if (ss[1] == 0) ss = primary_hostname;
  else if (Ustrcmp(ss, "@[]") == 0)
    {
    ip_address_item *ip;
    for (ip = host_find_interfaces(); ip != NULL; ip = ip->next)
      if (Ustrcmp(ip->address, cb->host_address) == 0) return OK;
    return FAIL;
    }
  }

/* If the pattern is an IP address, optionally followed by a bitmask count,
mask it and the current IP address, and do a binary comparison on them.
The function string_is_ip_address(), when given a non-NULL second argument,
puts the offset of a '/' in there and clobbers the '/' to a zero. */

if (string_is_ip_address(ss, &maskoffset))
  {
  int i;
  int address[4];
  int incoming[4];
  int mlen;
  int size = host_aton(ss, address);
  int insize;

  /* No mask => all bits to be checked */

  if (maskoffset == 0) mlen = 99999;    /* Big number */
  else
    {
    mlen = Uatoi(ss + maskoffset + 1);
    ss[maskoffset] = '/';    /* restore the slash */
    }

  /* Convert the incoming address to binary. */

  insize = host_aton(cb->host_address, incoming);

  /* Convert IPv4 addresses given in IPv6 compatible mode, which represent
  connections from IPv4 hosts to IPv6 hosts, that is, addresses of the form
  ::ffff:<v4address>, to IPv4 format. */

  if (insize == 4 && incoming[0] == 0 && incoming[1] == 0 &&
      incoming[2] == 0xffff)
    {
    insize = 1;
    incoming[0] = incoming[3];
    }

  /* No match if the sizes don't agree. */

  if (insize != size) return FAIL;

  /* Else do the masked comparison. */

  for (i = 0; i < size; i++)
    {
    int mask;
    if (mlen == 0) mask = 0;
    else if (mlen < 32)
      {
      mask = (-1) << (32 - mlen);
      mlen = 0;
      }
    else
      {
      mask = -1;
      mlen -= 32;
      }
    if ((incoming[i] & mask) != (address[i] & mask)) return FAIL;
    }
  return OK;
  }

/* If the item is of the form net24-dbm;/some/file then it is a file lookup
on a masked IP network, in textual form. */

if (Ustrncmp(ss, "net", 3) == 0 && (semicolon = Ustrchr(ss, ';')) != NULL)
  {
  int mlen = 0;
  for (t = ss + 3; isdigit(*t); t++) mlen = mlen * 10 + *t - '0';
  if (*t++ == '-')
    {
    int insize;
    int search_type;
    int incoming[4];
    void *handle;
    uschar *filename, *key;
    uschar buffer[64];

    /* If no mask was supplied, set a negative value */

    if (mlen == 0 && t == ss+4) mlen = -1;

    /* Find the search type */

    *semicolon = 0;
    search_type = search_findtype(t);
    *semicolon = ';';

    if (search_type < 0) log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s",
      search_error_message);

    /* Adjust parameters for the type of lookup. For a query-style
    lookup, there is no file name, and the "key" is just the query. For
    a single-key lookup, the key is the current IP address, masked
    appropriately, and reconverted to text form, with the mask appended. */

    if (mac_islookup(search_type, lookup_querystyle))
      {
      filename = NULL;
      key = semicolon + 1;
      }
    else
      {
      insize = host_aton(cb->host_address, incoming);
      host_mask(insize, incoming, mlen);
      (void)host_nmtoa(insize, incoming, mlen, buffer);
      key = buffer;
      filename = semicolon + 1;
      }

    /* Now do the actual lookup; note that there is no search_close() because
    of the caching arrangements. */

    handle = search_open(filename, search_type, 0, NULL, NULL);
    if (handle == NULL) log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s",
      search_error_message);
    if (search_find(handle, filename, key, -1, NULL) != NULL) return OK;
    return search_find_defer? DEFER: FAIL;
    }
  }

/* The pattern is not an IP address or network reference of any kind. That is,
it is a host name pattern. Check the characters of the pattern to see if they
comprise only letters, digits, full stops, and hyphens (the constituents of
domain names). Allow underscores, as they are all too commonly found. Sigh. */

for (t = ss; *t != 0; t++)
  if (!isalnum(*t) && *t != '.' && *t != '-' && *t != '_') break;

/* If the pattern is a complete domain name, with no fancy characters, look up
its IP address and match against that. Note that a multi-homed host will add
items to the chain. */

if (*t == 0)
  {
  host_item h;
  h.next = NULL;
  h.name = ss;
  h.address = NULL;
  if (host_find_byname(&h, NULL, NULL, FALSE, FALSE) != HOST_FIND_FAILED)
    {
    host_item *hh;
    for (hh = &h; hh != NULL; hh = hh->next)
      {
      if (Ustrcmp(hh->address, (Ustrchr(hh->address, ':') == NULL)?
        cb->host_ipv4 : cb->host_address) == 0)
          return OK;
      }
    }
  return FAIL;
  }

/* All subsequent comparisons require the host name, and can be done using the
general string matching function. When this function is called for outgoing
hosts, the name is always given explicitly. If it is NULL, it means we must use
sender_host_name and its aliases, looking them up if necessary. */

if (cb->host_name != NULL)   /* Explicit host name given */
  return match_check_string(cb->host_name, ss, -1, TRUE, TRUE, TRUE, NULL);

/* Host name not given; use sender host and its aliases */

if (sender_host_name == NULL)
  {
  HDEBUG(D_host_lookup)
    debug_printf("sender host name required, to match against %s\n", ss);
  if (host_lookup_failed || host_name_lookup() != OK) return ERROR;
  host_build_sender_fullhost();
  }

/* Match on the sender host name, using the general matching function */

switch(match_check_string(sender_host_name, ss, -1, TRUE, TRUE, TRUE, NULL))
  {
  case OK:    return OK;
  case DEFER: return DEFER;
  }

/* If there are aliases, try matching on them. */

aliases = sender_host_aliases;
while (*aliases != NULL)
  {
  switch(match_check_string(*aliases++, ss, -1, TRUE, TRUE, TRUE, NULL))
    {
    case OK:    return OK;
    case DEFER: return DEFER;
    }
  }
return FAIL;
}




/*************************************************
*    Check a specific host matches a host list   *
*************************************************/

/* This function is passed a host list containing items in a number of
different formats and the identity of a host. Its job is to determine whether
the given host is in the set of hosts defined by the list. The host name is
passed as a pointer so that it can be looked up if needed and not already
known. This is commonly the case when called from verify_check_host() to check
an incoming connection. When called from elsewhere the host name should usually
be set.

This function is now just a front end to match_check_list(), which runs common
code for scanning a list. We pass it the check_host() function to perform a
single test.

Arguments:
  listptr              pointer to the host list
  cache_bits           pointer to cache for named lists, or NULL
  host_name            the host name or NULL, implying use sender_host_name and
                         sender_host_aliases, looking them up if required
  host_address         the IP address

Returns:    OK    if the host is in the defined set
            FAIL  if the host is not in the defined set,
            DEFER if a data lookup deferred (not a host lookup)

If the host name was needed in order to make a comparison, and could not be
determined from the IP address, the result is FAIL unless the item
"+allow_unknown" was met earlier in the list, in which case OK is returned. */

int
verify_check_this_host(uschar **listptr, unsigned int *cache_bits,
  uschar *host_name, uschar *host_address)
{
unsigned int *local_cache_bits = cache_bits;
check_host_block cb;
cb.host_name = host_name;
cb.host_address = host_address;

/* If the host address starts off ::ffff: it is an IPv6 address in
IPv4-compatible mode. Find the IPv4 part for checking against IPv4
addresses. */

cb.host_ipv4 = (Ustrncmp(host_address, "::ffff:", 7) == 0)?
  host_address + 7 : host_address;

return match_check_list(listptr, 0, hostlist_anchor, &local_cache_bits,
  check_host, &cb, MCL_HOST,
  (host_address == sender_host_address)? US"host" : host_address);
}




/*************************************************
*      Check the remote host matches a list      *
*************************************************/

/* This is a front end to verify_check_this_host(), created because checking
the remote host is a common occurrence. With luck, a good compiler will spot
the tail recursion and optimize it. If there's no host address, this is
command-line SMTP input - check against an empty string for the address.

Arguments:
  listptr              pointer to the host list

Returns:               the yield of verify_check_this_host(),
                       i.e. OK, FAIL, or DEFER
*/

int
verify_check_host(uschar **listptr)
{
return verify_check_this_host(listptr, sender_host_cache, NULL,
  (sender_host_address == NULL)? US"" : sender_host_address);
}





/*************************************************
*    Invert an IP address for a DNS black list   *
*************************************************/

/*

Arguments:
  buffer         where to put the answer
  address        the address to invert
*/

static void
invert_address(uschar *buffer, uschar *address)
{
int bin[4];
uschar *bptr = buffer;

/* If this is an IPv4 address mapped into IPv6 format, adjust the pointer
to the IPv4 part only. */

if (Ustrncmp(address, "::ffff:", 7) == 0) address += 7;

/* Handle IPv4 address: when HAVE_IPV6 is false, the result of host_aton() is
always 1. */

if (host_aton(address, bin) == 1)
  {
  int i;
  int x = bin[0];
  for (i = 0; i < 4; i++)
    {
    sprintf(CS bptr, "%d.", x & 255);
    while (*bptr) bptr++;
    x >>= 8;
    }
  }

/* Handle IPv6 address. Actually, as far as I know, there are no IPv6 addresses
in any DNS black lists, and the format in which they will be looked up is
unknown. This is just a guess. */

#if HAVE_IPV6
else
  {
  int i, j;
  for (j = 3; j >= 0; j--)
    {
    int x = bin[j];
    for (i = 0; i < 8; i++)
      {
      sprintf(CS bptr, "%x.", x & 15);
      while (*bptr) bptr++;
      x >>= 4;
      }
    }
  }
#endif
}



/*************************************************
*        Check host against DNS black lists      *
*************************************************/

/* This function runs checks against a list of DNS black lists, until one
matches. Each item on the list can be of the form

  domain=ip-address/key

The domain is the right-most domain that is used for the query, for example,
blackholes.mail-abuse.org. If the IP address is present, there is a match only
if the DNS lookup returns a matching IP address. Several addresses may be
given, comma-separated, for example: x.y.z=127.0.0.1,127.0.0.2.

If no key is given, what is looked up in the domain is the inverted IP address
of the current client host. If a key is given, it is used to construct the
domain for the lookup. For example,

  dsn.rfc-ignorant.org/$sender_address_domain

After finding a match in the DNS, the domain is placed in $dnslist_domain, and
then we check for a TXT record for an error message, and if found, save its
value in $dnslist_text. We also cache everything in a tree, to optimize
multiple lookups.

Note: an address for testing RBL is 192.203.178.39
Note: an address for testing DUL is 192.203.178.4
Note: a domain for testing RFCI is example.tld.dsn.rfc-ignorant.org

Arguments:
  listptr      the domain/address/data list

Returns:    OK      successful lookup (i.e. the address is on the list), or
                      lookup deferred after +include_unknown
            FAIL    name not found, or no data found for the given type, or
                      lookup deferred after +exclude_uknown (default)
            DEFER   lookup failure, if +defer_unknown was set
*/

int
verify_check_dnsbl(uschar **listptr)
{
int sep = 0;
int defer_return = FAIL;
uschar *list = *listptr;
uschar *domain;
uschar buffer[1024];
uschar query[256];         /* DNS domain max length */
uschar revadd[128];        /* Long enough for IPv6 address */

/* Indicate that the inverted IP address is not yet set up */

revadd[0] = 0;

/* Loop through all the domains supplied, until something matches */

while ((domain = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
  {
  BOOL frc;
  dns_answer dnsa;
  dns_scan dnss;
  uschar *iplist;
  uschar *key;
  tree_node *t;
  dnsbl_cache_block *cb;

  HDEBUG(D_dnsbl) debug_printf("DNS list check: %s\n", domain);

  /* Deal with special values that change the behaviour on defer */

  if (domain[0] == '+')
    {
    if      (strcmpic(domain, US"+include_unknown") == 0) defer_return = OK;
    else if (strcmpic(domain, US"+exclude_unknown") == 0) defer_return = FAIL;
    else if (strcmpic(domain, US"+defer_unknown") == 0)   defer_return = DEFER;
    else
      log_write(0, LOG_MAIN|LOG_PANIC, "unknown item in dnslist (ignored): %s",
        domain);
    continue;
    }

  /* See if there's explicit data to be looked up */

  key = Ustrchr(domain, '/');
  if (key != NULL) *key++ = 0;

  /* See if there's a list of addresses supplied after the domain name */

  iplist = Ustrchr(domain, '=');
  if (iplist != NULL) *iplist++ = 0;

  /* Construct the query by adding the domain onto either the sending host
  address, or the given key string. */

  if (key == NULL)
    {
    if (sender_host_address == NULL) return FAIL;    /* can never match */
    if (revadd[0] == 0) invert_address(revadd, sender_host_address);
    frc = string_format(query, sizeof(query), "%s%s", revadd, domain);
    }
  else
    {
    frc = string_format(query, sizeof(query), "%s.%s", key, domain);
    }

  if (!frc)
    {
    log_write(0, LOG_MAIN|LOG_PANIC, "dnslist query is too long "
      "(ignored): %s...", query);
    continue;
    }

  /* Look for this query in the cache. */

  t = tree_search(dnsbl_cache, query);

  /* If not cached from a previous lookup, we must do a DNS lookup */

  if (t == NULL)
    {
    /* In case this is the first time the DNS resolver is being used. */

    dns_init(FALSE, FALSE);

    /* Set up a tree entry to cache the lookup */

    t = store_malloc(sizeof(tree_node) + Ustrlen(query));
    Ustrcpy(t->name, query);
    t->data.ptr = cb = store_malloc(sizeof(dnsbl_cache_block));
    (void)tree_insertnode(&dnsbl_cache, t);

    /* Do the DNS loopup . */

    HDEBUG(D_dnsbl) debug_printf("new DNS lookup for %s\n", query);
    cb->rc = dns_basic_lookup(&dnsa, query, T_A);
    cb->text_set = FALSE;

    /* If the lookup succeeded, cache the RHS address. The code allows for
    more than one address - this was for complete generality and the possible
    use of A6 records. However, A6 records have been reduced to experimental
    status (August 2001) and may die out. So they may never get used at all,
    let alone in dnsbl records. However, leave the code here, just in case. */

    if (cb->rc == DNS_SUCCEED)
      {
      dns_record *rr;
      for (rr = dns_next_rr(&dnsa, &dnss, RESET_ANSWERS);
	   rr != NULL;
	   rr = dns_next_rr(&dnsa, &dnss, RESET_NEXT))
	{
        if (rr->type == T_A)
          {
          cb->rhs = dns_address_from_rr(&dnsa, rr);
          break;
          }
	}
      }
    }

  /* Previous lookup was cached */

  else
    {
    HDEBUG(D_dnsbl) debug_printf("using result of previous DNS lookup\n");
    cb = t->data.ptr;
    }

  /* We now have the result of the DNS lookup, either newly done, or cached
  from a previous call. If the lookup succeeded, check against the address
  list. */

  if (cb->rc == DNS_SUCCEED)
    {
    if (iplist != NULL)
      {
      int ipsep = ',';
      uschar ip[46];
      uschar *ptr = iplist;
      dns_address *da = NULL;

      while (string_nextinlist(&ptr, &ipsep, ip, sizeof(ip)) != NULL)
        {
        for (da = cb->rhs; da != NULL; da = da->next)
	  { if (Ustrcmp(CS da->address, ip) == 0) break; }
        if (da != NULL) break;
        }

      /* No IP address matched any of those in the list. In that case,
      behave as if the DNSBL lookup had not succeeded, i.e. the host is
      not on the list. */

      if (da == NULL)
        {
        HDEBUG(D_dnsbl)
	  {
	  debug_printf("DNS lookup for %s suceeded\n", query);
	  debug_printf("=> but we are not accepting this block class because\n");
          debug_printf("=> there was no match for %s\n", iplist);
	  }
        continue;   /* With next DNSBL domain */
        }
      }

    /* Either there was no IP list, or the record matched. Look up a TXT record
    if it hasn't previously been done. */

    if (!cb->text_set)
      {
      cb->text_set = TRUE;
      if (dns_basic_lookup(&dnsa, query, T_TXT) == DNS_SUCCEED)
        {
        dns_record *rr;
        for (rr = dns_next_rr(&dnsa, &dnss, RESET_ANSWERS);
             rr != NULL;
             rr = dns_next_rr(&dnsa, &dnss, RESET_NEXT))
          if (rr->type == T_TXT) break;
        if (rr != NULL)
          {
          int len = (rr->data)[0];
          if (len > 511) len = 127;
          cb->text = string_sprintf("%.*s", len, (const uschar *)(rr->data+1));
          }
        }
      else cb->text = NULL;
      }

    HDEBUG(D_dnsbl)
      {
      debug_printf("DNS lookup for %s succeeded\n", query);
      debug_printf("=> that means %s is listed at %s\n",
        (key == NULL)? sender_host_address : key, domain);
      }

    dnslist_domain = string_copy(domain);
    dnslist_value = cb->rhs->address;
    dnslist_text = cb->text;
    return OK;
    }

  /* There was a problem with the DNS lookup */

  if (cb->rc != DNS_NOMATCH && cb->rc != DNS_NODATA)
    {
    log_write(L_dnslist_defer, LOG_MAIN,
      "DNS list lookup defer (probably timeout) for %s: %s", query,
      (defer_return == OK)?   US"assumed in list" :
      (defer_return == FAIL)? US"assumed not in list" :
                              US"returned DEFER");
    return defer_return;
    }

  /* No entry was found in the DNS; continue for next domain */

  HDEBUG(D_dnsbl)
    {
    debug_printf("DNS lookup for %s failed\n", query);
    debug_printf("=> that means %s is not listed at %s\n",
      (key == NULL)? sender_host_address : key, domain);
    }
  }      /* Continue with next domain */

return FAIL;
}

/* End of verify.c */
